/*
  Copyright (C) 2013-2019 Expedia Inc.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.hotels.styx.common;

import static java.util.Objects.requireNonNull;

/**
 * Tuple of two values.
 *
 * @param <K> value 1
 * @param <V> value 2
 */
public class Pair<K, V> {
    private final K key;
    private final V value;

    private Pair(K key, V value) {
        this.key = requireNonNull(key);
        this.value = requireNonNull(value);
    }

    public static <K, V> Pair<K, V> pair(K key, V value) {
        return new Pair<>(key, value);
    }

    public K key() {
        return key;
    }

    public V value() {
        return value;
    }

    @Override
    public String toString() {
        return "[" + key + "," + value + "]";
    }
}
